# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-08 13:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_courier', '0002_site_contacts'),
    ]

    operations = [
        migrations.CreateModel(
            name='FailedMessage',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True, serialize=False,
                    verbose_name='ID')),
                ('backend', models.CharField(max_length=100)),
                ('address', models.CharField(max_length=500)),
                ('protocol', models.CharField(max_length=100)),
                ('message', models.TextField()),
                ('error', models.TextField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'failed message',
                'default_permissions': (),
            },
        ),
        migrations.AlterField(
            model_name='template',
            name='send_to_site_contacts',
            field=models.BooleanField(
                default=False,
                help_text='Whether this message is sent to the site contacts '
                          'or to the notification recipient.'),
        ),
    ]
