<h1>{{ article.title }}</h1>
<p>By: {{ article.author }}</p>
<p>{{ article.content }}</p>
{% if comments %}
    <h2>Comments</h2>
    {% for comment in comments %}
        <div><a href="mailto:{{ comment.poster.email }}">{{ comment.poster.name }}</a></div>
        <p>{{ comment.content }}</p>
    {% endfor %}
    </ul>
{% else %}
{% endif %}
{% if token %}
    <h2>Add a comment</h2>
    <form action="{% url 'tests:comment' article.id %}" method="post">
        {% csrf_token %}
        <input type="hidden" name="token" value="{{ token }}"/>
        <label for="content">Content</label><input type="text" id="content" name="content"/>
        <input type="submit" value="Comment"/>
    </form>
{% endif %}